// The MIT License

// Copyright (c) 2009 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

import Connectable::*;
import GetPut::*;
import ClientServer::*;

//AWB includes
`include "asim/provides/low_level_platform_interface.bsh"
`include "asim/provides/soft_connections.bsh"
`include "asim/provides/common_services.bsh"

// Local includes
`include "asim/provides/audio_processor_types.bsh"
`include "asim/provides/audio_pipeline.bsh"

`include "asim/rrr/remote_client_stub_AUDIOPROCESSORRRR.bsh"
`include "asim/rrr/remote_server_stub_AUDIOPROCESSORRRR.bsh"


module [CONNECTED_MODULE] mkConnectedApplication ();

   // Instantiate the rrr stubs
   ClientStub_AUDIOPROCESSORRRR client_stub <- mkClientStub_AUDIOPROCESSORRRR();
   ServerStub_AUDIOPROCESSORRRR server_stub <- mkServerStub_AUDIOPROCESSORRRR();

   // Instantiate the audio pipeline
   AudioPipeline pipeline <- mkAudioPipeline();

   rule feedInput;
     let command <- server_stub.acceptRequest_SendUnprocessedStream();
     AudioProcessorControl ctrl = unpack(truncate(command.ctrl));

     if(ctrl == EndOfFile)
       begin
         pipeline.sampleInput.put(tagged EndOfFile);
       end
     else 
       begin
         pipeline.sampleInput.put(tagged Sample unpack(truncate(command.sample)));
       end
   endrule

   rule feedOutput;
     let pipelineData <- pipeline.sampleOutput.get();
     AudioProcessorControl endOfFileTag = EndOfFile;
     AudioProcessorControl sampleTag = Data;

     case (pipelineData) matches
       tagged EndOfFile: client_stub.makeRequest_SendProcessedStream(zeroExtend(pack(endOfFileTag)),?);
       tagged Sample .sample:client_stub.makeRequest_SendProcessedStream(zeroExtend(pack(sampleTag)),
                                                                         zeroExtend(pack(sample)));
     endcase
   endrule

endmodule


